local uabPerlClassicFrameGroup = uabFrameGroup:NewModule("PerlClassic")

uabPerlClassicFrameGroup.controls = {}
uabPerlClassicFrameGroup.defaultDB = {
    framegroups = {
        ["Player"] = {
            expression = "^Perl_Player_.+_CastClickOverlay$",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "player",
        },
        ["Player Combat"] = {
            expression = "^Perl_CombatDisplay_[^T].+_CastClickOverlay$",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "player",
        },
        ["Player Pet"] = {
            expression = "^Perl_Player_Pet_.+_CastClickOverlay$",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "pet",
        },
        ["Party"] = {
            expression = "^Perl_Party_MemberFrame%d+_.+_CastClickOverlay$",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "partyraid",
        },
        ["Focus"] = {
            expression = "^Perl_Focus_.+_CastClickOverlay$",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "focus",
        },
        ["Target"] = {
            expression = "^Perl_Target_[^T].+_CastClickOverlay$",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "target",
        },
        ["Target Combat"] = {
            expression = "^Perl_CombatDisplay_Target_.+_CastClickOverlay$",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "target",
        },
        ["Target of Target"] = {
            expression = "^Perl_Target_Target_[^T].+_CastClickOverlay$",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "targettarget",
        },
        ["ToT Target"] = {
            expression = "^Perl_Target_Target_Target_.+_CastClickOverlay$",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "targettargettarget",
        },
    },
}

function uabPerlClassicFrameGroup:OnInitialize()
    self.super.OnInitialize(self)
end

function uabPerlClassicFrameGroup:OnEnable()
end

function uabPerlClassicFrameGroup:OnDisable()
end

function uabPerlClassicFrameGroup:NewActivator(activator)
	self.controls[activator] = {}
    
    self:AllocateControlsForUnitGroup(activator, "player")
    self:AllocateControlsForUnitGroup(activator, "pet")
    self:AllocateControlsForUnitGroup(activator, "partyraid")
    self:AllocateControlsForUnitGroup(activator, "focus")
    self:AllocateControlsForUnitGroup(activator, "focustarget")
    self:AllocateControlsForUnitGroup(activator, "target")
    self:AllocateControlsForUnitGroup(activator, "targettarget")
    self:AllocateControlsForUnitGroup(activator, "targettargettarget")
end

function uabPerlClassicFrameGroup:AllocateControlsForUnitGroup(activator, groupname)
    self.controls[activator][groupname] = {}
    
	for idx = 1, uab.constants.groupInfo[groupname].unitcount do
        table.insert(self.controls[activator][groupname], activator:NewUnitGroupControl(groupname))
        table.insert(self.controls[activator][groupname], activator:NewUnitGroupControl(groupname))
        table.insert(self.controls[activator][groupname], activator:NewUnitGroupControl(groupname))
        table.insert(self.controls[activator][groupname], activator:NewUnitGroupControl(groupname))
        table.insert(self.controls[activator][groupname], activator:NewUnitGroupControl(groupname))
        table.insert(self.controls[activator][groupname], activator:NewUnitGroupControl(groupname))
        table.insert(self.controls[activator][groupname], activator:NewUnitGroupControl(groupname))
        table.insert(self.controls[activator][groupname], activator:NewUnitGroupControl(groupname))
        table.insert(self.controls[activator][groupname], activator:NewUnitGroupControl(groupname))
        table.insert(self.controls[activator][groupname], activator:NewUnitGroupControl(groupname))
    end
end

function uabPerlClassicFrameGroup:HaveFrameControl(frame, activator)
	local _, options = self:OptionsForFrame(frame)
	
	if not options then return end

	if self.controls[activator] and self.controls[activator][options.unitgroup] then
		return # self.controls[activator][options.unitgroup] > 0
	end
end

function uabPerlClassicFrameGroup:NextFrameControl(frame, activator)
	local _, options = self:OptionsForFrame(frame)

	if not options then return end

	local control
	
    if self.controls[activator] and self.controls[activator][options.unitgroup]
        and # self.controls[activator][options.unitgroup] > 0 then
		control = table.remove(self.controls[activator][options.unitgroup])
	end

	return control
end

function uabPerlClassicFrameGroup:OptionsForFrame(frame)
    for name, defn in pairs(self.db.profile.framegroups) do
        if self:MatchFrame(frame, defn) then
            return name, defn
        end
    end
end

function uabPerlClassicFrameGroup:CreateFrameGroupOptions()
    local args = {}

    args = {
        type = 'group',
        name = "Perl Classic",
        desc = "Use, anchor, scale, and match expression for Perl Classic frames",
        args = {
            enabled = {
                type = 'toggle',
                name = "Enabled",
                desc = "When checked, enable the use of UAB action sets for Perl Classic frames",
                order = 1,
                get = function()
                    return not self.db.profile.disabled
                end,
                set = function(v)
                    self.db.profile.disabled = not v
                    self.core.core.main:FrameGroupsChanged()
                end,
            },
            
            spacing3 = {
                name = " ",
                type = 'header',
                order = 5,
            },
        
        },
    }
    
    args.args["Player"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Player"], "Player", "Perl Classic Player")
    args.args["Player"].disabled = function() return self.db.profile.disabled end
    args.args["Player"].order = 10
    
    args.args["Player Combat"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Player Combat"], "Player Combat", "Perl Classic Player Combat")
    args.args["Player Combat"].disabled = function() return self.db.profile.disabled end
    args.args["Player Combat"].order = 20
    
    args.args["Player Pet"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Player Pet"], "Player Pet", "Perl Classic Player Pet")
    args.args["Player Pet"].disabled = function() return self.db.profile.disabled end
    args.args["Player Pet"].order = 30
    
    args.args["Party"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Party"], "Party", "Perl Classic Party")
    args.args["Party"].disabled = function() return self.db.profile.disabled end
    args.args["Party"].order = 40
    
    args.args["Focus"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Focus"], "Focus", "Perl Classic Focus")
    args.args["Focus"].disabled = function() return self.db.profile.disabled end
    args.args["Focus"].order = 50
    
    args.args["Target"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Target"], "Target", "Perl Classic Target")
    args.args["Target"].disabled = function() return self.db.profile.disabled end
    args.args["Target"].order = 60
    
    args.args["Target Combat"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Target Combat"], "Target Combat", "Perl Classic Target Combat")
    args.args["Target Combat"].disabled = function() return self.db.profile.disabled end
    args.args["Target Combat"].order = 70
    
    args.args["Target of Target"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Target of Target"], "Target of Target", "Perl Classic Target of Target")
    args.args["Target of Target"].disabled = function() return self.db.profile.disabled end
    args.args["Target of Target"].order = 80
    
    args.args["ToT Target"] = self:CreateFrameGroupOption(self.db.profile.framegroups["ToT Target"], "ToT Target", "Perl Classic ToT Target")
    args.args["ToT Target"].disabled = function() return self.db.profile.disabled end
    args.args["ToT Target"].order = 90
    
    return args
end

function uabPerlClassicFrameGroup:SetupLayouts()
    uabLayoutManager:LayoutChanged(self.db.profile.framegroups["Player"].layout, self, "Player")
    uabLayoutManager:LayoutChanged(self.db.profile.framegroups["Player Combat"].layout, self, "Player Combat")
    uabLayoutManager:LayoutChanged(self.db.profile.framegroups["Player Pet"].layout, self, "Player Pet")
    uabLayoutManager:LayoutChanged(self.db.profile.framegroups["Party"].layout, self, "Party")
    uabLayoutManager:LayoutChanged(self.db.profile.framegroups["Focus"].layout, self, "Focus")
    uabLayoutManager:LayoutChanged(self.db.profile.framegroups["Target"].layout, self, "Target")
    uabLayoutManager:LayoutChanged(self.db.profile.framegroups["Target Combat"].layout, self, "Target Combat")
    uabLayoutManager:LayoutChanged(self.db.profile.framegroups["Target of Target"].layout, self, "Target of Target")
    uabLayoutManager:LayoutChanged(self.db.profile.framegroups["ToT Target"].layout, self, "ToT Target")
end

function uabPerlClassicFrameGroup:SetLayoutConfigureOptions(name, args)
    self.core.core.options.args.lookandfeel.args.framegroups.args[self.name].args[name].args.layout.args.configure.args = args
end

